package droid64.d64;

public class D67Constants {

	/** Directory track sector sequence */
	protected static final int[]  DIR_SECTORS = {
			1, 4, 7, 10, 13, 16,
			2, 5, 8, 11, 14, 17,
			3, 6, 9, 12, 15, 18 };

	/** Table for finding track/sector of CP/M allocation unit */
	protected static final int[][] CPM_ZONES = {
			// FirstTrack, SectorsPerTrack, SectorsInZone, ReservedZoneSectors
			{ 1, 21, 357, 2},
			{18, 19, 133, 1},
			{25, 18, 108, 0},
			{31, 17,  85, 0}
	};

	/** CP/M C128 D67 Directory track */
	protected static final int C128_SS_DIR_TRACK = 1;
	/** CP/M C128 D67 Directory sectors */
	protected static final int[] C128_SS_DIR_SECTORS = { 10, 15, 20, 4, 9, 14, 19, 3 };
	/** CP/M C64 D67 Directory track */
	protected static final int C64_SS_DIR_TRACK = 3;
	/** CP/M C64 D67 Directory sectors */
	protected static final int[] C64_SS_DIR_SECTORS = { 1, 2, 3, 4, 5, 6 ,7 };

	/** Offsets of tracks */
	protected static final CbmTrack[] D67_TRACKS = {
			new CbmTrack(21,   0, 0x000000 ),		//   0 (dummy)
			new CbmTrack(21,   0, 0x000000 ),       //   1
			new CbmTrack(21,  21, 0x001500 ),       //   2
			new CbmTrack(21,  42, 0x002a00 ),       //   3
			new CbmTrack(21,  63, 0x003f00 ),       //   4
			new CbmTrack(21,  84, 0x005400 ),       //   5
			new CbmTrack(21, 105, 0x006900 ),       //   6
			new CbmTrack(21, 126, 0x007e00 ),       //   7
			new CbmTrack(21, 147, 0x009300 ),       //   8
			new CbmTrack(21, 168, 0x00a800 ),       //   9
			new CbmTrack(21, 189, 0x00bd00 ),       //  10
			new CbmTrack(21, 210, 0x00d200 ),       //  11
			new CbmTrack(21, 231, 0x00e700 ),       //  12
			new CbmTrack(21, 252, 0x00fc00 ),       //  13
			new CbmTrack(21, 273, 0x011100 ),       //  14
			new CbmTrack(21, 294, 0x012600 ),       //  15
			new CbmTrack(21, 315, 0x013b00 ),       //  16
			new CbmTrack(21, 336, 0x015000 ),       //  17
			new CbmTrack(20, 357, 0x016500 ),       //  18
			new CbmTrack(20, 377, 0x017900 ),       //  19
			new CbmTrack(20, 397, 0x018d00 ),       //  20
			new CbmTrack(20, 417, 0x01a100 ),       //  21
			new CbmTrack(20, 437, 0x01b500 ),       //  22
			new CbmTrack(20, 457, 0x01c900 ),       //  23
			new CbmTrack(20, 477, 0x01dd00 ),       //  24
			new CbmTrack(18, 497, 0x01f100 ),       //  25
			new CbmTrack(18, 515, 0x020300 ),       //  26
			new CbmTrack(18, 533, 0x021500 ),       //  27
			new CbmTrack(18, 551, 0x022700 ),       //  28
			new CbmTrack(18, 569, 0x023900 ),       //  29
			new CbmTrack(18, 587, 0x024b00 ),       //  30
			new CbmTrack(17, 605, 0x025d00 ),       //  31
			new CbmTrack(17, 622, 0x026e00 ),       //  32
			new CbmTrack(17, 639, 0x027f00 ),       //  33
			new CbmTrack(17, 656, 0x029000 ),       //  34
			new CbmTrack(17, 673, 0x02a100 ),       //  35
			new CbmTrack(17, 690, 0x02b200 ),       //  36
			new CbmTrack(17, 707, 0x02c300 ),       //  37
			new CbmTrack(17, 724, 0x02d400 ),       //  38
			new CbmTrack(17, 741, 0x02e500 ),       //  39
			new CbmTrack(17, 758, 0x02f600 ),       //  40
	};

	protected static final short[] NEWD67DATA = {
			0x12, 0x01, 0x41, 0x00, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f,	//00016500 (dec = 91392)
			0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f,	//00016510
			0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f,	//00016520
			0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f,	//00016530
			0x15, 0xff, 0xff, 0x1f, 0x15, 0xff, 0xff, 0x1f, 0x11, 0xfc, 0xff, 0x07, 0x13, 0xff, 0xff, 0x07,	//00016540
			0x13, 0xff, 0xff, 0x07, 0x13, 0xff, 0xff, 0x07, 0x13, 0xff, 0xff, 0x07, 0x13, 0xff, 0xff, 0x07,	//00016550
			0x13, 0xff, 0xff, 0x07, 0x12, 0xff, 0xff, 0x03, 0x12, 0xff, 0xff, 0x03, 0x12, 0xff, 0xff, 0x03,	//00016560
			0x12, 0xff, 0xff, 0x03, 0x12, 0xff, 0xff, 0x03, 0x12, 0xff, 0xff, 0x03, 0x11, 0xff, 0xff, 0x01,	//00016570
			0x11, 0xff, 0xff, 0x01, 0x11, 0xff, 0xff, 0x01, 0x11, 0xff, 0xff, 0x01, 0x11, 0xff, 0xff, 0x01,	//00016580
			0x31, 0x32, 0x33, 0x34, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0,	//00016590
			0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00,	//000165a0
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	//000165b0
			0x00, 0x00, 0x00, 0x00, 0x00 ,0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	//000165c0
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	//000165d0
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	//000165e0
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	//000165f0
			0x00, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00	//00016600
	};

	private D67Constants() {
		// Not used
	}
}
